;======================== header ===============================================
;+
; NAME:
;   date_selector
; PURPOSE:
;   standardised way for the SWBS to handle input date information and calculate a
;   few time-dependent variables
; CATEGORY:
;   SWB version 1.00
; CALLING SEQUENCE:
;   (meant to be used in batchmode)
; INPUTS:
;
; KEYWORD PARAMETERS:
;   date      = if date is an integer, it is the number of days we need to backtrace from 'now'.
;             = if date is a simple string, then it is the starting date  (as readible by anytim2jd)
;                                       from which processing starts untill 'now'
;             = if date is an array of two string elements, then the first element is the starting date
;                                       and the second element is the ending date 
; OUTPUTS:
;   
; COMMON BLOCKS:
;   none
; SIDE EFFECTS:
;   unknown
; RESTRICTIONS:
;   unknown
; PROCEDURE:
;    date_selector, date=['23-APR-04', '28-APR-04'], days1=days1, days2=days2, 
;                                                 days3=days3, solar_r_arcsec=r, solarb0=b0
;    date_selector, date=2, days1=days1, days2=days2, days3=days3
; CALLS:
;
;
; TO DO:
;
; MODIFICATION HISTORY:
;   2004/12/13, ver 1.00, David Berghmans
;   2005/06/09, ver 1.01, Thanassis Katsiyannis, logfile capability
;                         added.
;-
;======================== end header ===========================================


PRO date_selector, date=date, logfilename=logfilename, $ ; input
     days1=t, days2=refdate1, days3=refdate2, solar_r_arcsec=solar_r_arcsec, solarb0=solarb0   ; output

    date_type=size(date, /structure)
    CASE date_type.type_name OF
        'INT': IF date_type.n_elements eq 1 THEN nrdays=date ELSE stop
     'STRING':  IF date_type.n_elements eq 1 THEN startdate=date $
                            ELSE IF date_type.n_elements eq 2 THEN BEGIN
			                                             startdate=date[0] 
								     enddate=date[1]
								   ENDIF
      ELSE: BEGIN
                print, 'input data type is not of appropiate formate'
                print, 'falling back on default nrdays=1 (selecting data of today)'
                nrdays=1				
             END 
     ENDCASE
     
     IF n_elements(enddate) NE 0 THEN t1=anytim2jd(enddate) $
                                 ELSE t1=anytim2jd(!stime)
     IF n_elements(nrdays) EQ 0 THEN nrdays=1
     IF n_elements(startdate) NE 0 THEN t0=anytim2jd(startdate)  $
                                   ELSE BEGIN
				     t0=t1
				     t0.int=t1.int-(nrdays-1)
				   ENDELSE

     t0= DOUBLE(t0.int) +ROUND(t0.frac) ; format that CALDAT can handle
     t1= DOUBLE(t1.int) +ROUND(t1.frac) ; format that CALDAT can handle
     nrdays=t1-t0+1 ; just in case somebody gives a nrdays that is not enddate-startdate
    
     t=t0+dindgen(nrdays)
      
     caldat, t, month, day, year  
     moy     = STRMID(STRTRIM(month+100,2),1,2)
     dom     = STRMID(STRTRIM(day+100,2),1,2)
     year_4ch= STRTRIM(year,2)      ;  eg '2004'
     year_2ch= STRMID(year_4ch,2,2) ;  eg '04'
     date_for_get_rb0p=year_2ch+'/'+moy+'/'+dom+', 1200:00.000'     ; noon today, format for get_rb0p
     refdate1= year_4ch+'/'+moy+'/'+dom+'/' ; something like '2004/09/01/'
     refdate2= year_4ch+moy+dom ; something like '20040901'
     
     
     solar_r_arcsec =DBLARR(nrdays)
     solarb0        =DBLARR(nrdays)
     FOR ithday=0, nrdays-1 DO BEGIN
       rb0p                   =get_rb0p(date_for_get_rb0p[ithday], /deg)
       solar_r_arcsec[ithday]=rb0p[0]
       solarb0[ithday]        =rb0p[1]
     ENDFOR     

END
