;======================== header ===============================================
;+
; NAME:
;   solarsoft_latest_events
; PURPOSE:
;   make zip files for Solar Weather Browser based on solarsoft latest events page
; CATEGORY:
;   SWB version 1.00
; CALLING SEQUENCE:
;   (meant to be used in batchmode)
; INPUTS:
;
;
; KEYWORD PARAMETERS:
;   /logfile: to print diagnostics to a logfile (default=stdout=screen)
;   date: date should be in the format '1970-01-02'
;         ATTENTION: date MUST exist in web site:
;         http://www.lmsal.com/solarsoft/latest_events/index.html
;        
;
; OUTPUTS:
;   
; COMMON BLOCKS:
;   none
; SIDE EFFECTS:
;   unknown
; RESTRICTIONS:
;   unknown
; PROCEDURE:
;
; CALLS:
;
;
; TO DO:
;
; MODIFICATION HISTORY:
;   2004/03/09, ver 1.00, David Berghmans
;   2005/08/12, ver 1.10, Thanassis Katsiyannis 
;       (T.Katsiyannis@oma.be), added date keyword, fixed bugs
;   2006/04/06, ver 2.00, Ingo Baumann
;       ;       overwriting of text is avoided 
;   2006/06/09, ver 2.00, Ingo Baumann
;       svg-information included
;   2006/07/11: C. Marque. add smf2svg to take into account new version of write2zip
;	       make tmpdir default directory outpur for temporary files
;   2006/10/06 C.Marque, add set_swbs and change code accordingly
;-
;-
;======================== end header ===========================================
PRO solarsoft_latest_events, logfile=logfile, crontab=crontab, startdate=startdate, enddate=enddate

; ------------------------ preamble -----------------------------------
 ; Who am I?
   myname   ='solarsoft_latest_events'

 ; main input/output
   set_swbs 
 ; log file control
   logfilename=!swbs.log_dir+myname
   IF keyword_set(logfile) THEN openw, log, logfilename, /get_lun,/append $
                           ELSE log=-1
    	    ; if /logfile keyword is set, open the logfile, if not
	    ; send output to stout (typically the screen).

 ; say hello
   printf, log, ' >>>> '+myname+'.pro started at '+ systime()

 ; keyword checking
 ;  ...

; ------------------------main code ------------------------------------

 ; main parameters
   datadir = !swbs.data_dir+'SLE/'
   tmpdir=!swbs.tmp_dir
   
;----------------------------------------------------------- 
 ; get reference information
   outdim=!swbs.outdim
   ref_solradius=!swbs.ref_solradius
   ref_xcenter=!swbs.ref_xcenter
   ref_ycenter=!swbs.ref_ycenter
;----------------------------------------------------------- 

   IF keyword_set(startdate) THEN BEGIN
       julstart=JULDAY(FIX(STRMID(startdate, 5, 2)),FIX(STRMID(startdate, 8, 2)),FIX(STRMID(startdate, 0, 4)))
       if not keyword_set(enddate) THEN enddate=startdate
       julend=JULDAY(FIX(STRMID(enddate, 5, 2)),FIX(STRMID(enddate, 8, 2)),FIX(STRMID(enddate, 0, 4)))
   ENDIF ELSE BEGIN
   ; get today
       julstart = round(systime(0,/JULIAN))
       julend   = julstart
   ENDELSE

   FOR jul_k=julstart,julend,1 DO BEGIN

       cd, datadir

       caldat, jul_k+1, mplus1, dplus1, yplus1
       caldat, jul_k, moy, dom, year
       moy     = STRMID(STRTRIM(moy+100,2),1,2)
       mplus1  = STRMID(STRTRIM(mplus1+100,2),1,2)
       dom     = STRMID(STRTRIM(dom+100,2),1,2)
       dplus1  = STRMID(STRTRIM(dplus1+100,2),1,2)
       year_4ch= STRTRIM(year,2) ;  eg '2004'
       yplus1  = STRTRIM(yplus1,2) ;  eg '2004'
       year_2ch= STRMID(year_4ch,2,2) ;  eg '04'

   
       noonofday=year_2ch+'/'+moy+'/'+dom+', 1200:00.000' ; format for get_rb0p
       refdate= year_4ch+'/'+moy+'/'+dom ; to check the date of the events
       refdate1= year_4ch+'-'+moy+'-'+dom+'T12:00:00.000Z' ; noon of day, format for anytim2jd

                                ; make output parameters
       date_suffix=year_2ch+moy+dom+'.120000'
       outdir=!swbs.out_dir+year_4ch+'/'+moy+'/'+dom+'/'
  
                                ; get solar parameters
       rb0p=get_rb0p(noonofday, /deg)
       solarb0=rb0p[1]
;----------------------------------------------------------- 
; define solar map

       outdim=512
       scale=outdim/1024.
       IF not keyword_set(crontab) THEN window, xs=outdim, ys=outdim ELSE BEGIN
           printf, log, '   Working in batchmode, using Z-buffer'
           set_plot, 'Z'
           device, set_resolution = [outdim,outdim] ; size of image
           device, z_buffering    = 0 ; 0 for 2D
       ENDELSE
       
                                ; calculate margins for grid overlay
       xmargin_pix= [ref_xcenter-ref_solradius,outdim-ref_solradius-ref_xcenter]
       ymargin_pix= [ref_ycenter-ref_solradius,outdim-ref_solradius-ref_ycenter]
       csx=FLOAT(!D.X_CH_SIZE)  ; 6.(for window) or 8(for z-buffer)
       csy=FLOAT(!D.Y_CH_SIZE)  ; 10.(for window) or 13.(for z-buffer)
                                ;csx=x size of characters in pix, csy=y size of characters in pix
                                ;csx and csy are the units for the x-margin and y-margin of map_set
                                ;their values are different for window and for z-buffer

       map_set, /orthographic, solarb0, 0, xmargin=xmargin_pix/csx, ymargin=ymargin_pix/csy, /noborder

;----------------------------------------------------------- 

       x       = intarr(100)
       y       = intarr(100)
       x_heliographic = intarr(100)
       y_heliographic = intarr(100)
       String1 = strarr(100)
       String2 = strarr(100)

       spawn,'rm -f index.html'

       IF NOT keyword_set(startdate) THEN BEGIN
       
;--get the latest events index.html-------------------------------- 
              
           spawn,'lynx -dump -nolist http://www.lmsal.com/solarsoft/latest_events/index.html > ' $
             +datadir+'/index.html'
              
       ENDIF ELSE BEGIN


;--get archive events index.html-------------------------------- 

          ; first get link

          spawn,'wgetrwc -t1 -l1 -nd -N --no-parent -A -r http://www.lmsal.com/solarsoft/latest_events_archive.html'
          spawn,'grep "last_events_'+yplus1+mplus1+dplus1+'" latest_events_archive.html | sed -e ''s/^.*HREF="//''| sed -e ''s/">*.*$//'' > temp1'
          linkadd=''
          openr,u,'temp1',/get_LUN
          IF EOF(u) then goto, jump
          readf,u,linkadd
          close,u
          free_LUN,u
          spawn,'rm -f temp1'

          ; now download webpage
          spawn,'lynx -dump -nolist http://www.lmsal.com/solarsoft/'+linkadd+' > '+datadir+'/index.html'

      ENDELSE


;--read-in the file--------------------------------------------- 

      indexfile=datadir+'index.html'
      
      htmlfile='' & line=''
      openr, unit, indexfile, /get_lun
      WHILE not eof(unit) DO BEGIN
          readf, unit, line
          htmlfile=[htmlfile, line]
      ENDWHILE
      close, unit
      free_lun, unit
      nlines=n_elements(htmlfile)

;----------------------------------------------------------- 
      event_begin=''
      event_end=''
      event_peak=''
      event_mag=''
      event_loc=''
      event_date=''
       
      j=0
      EW=''
      NS=''

      FOR i=1,nlines-1,1 DO BEGIN
           
          if STRMATCH(strmid(htmlfile[i],5,1,/reverse_offset), 'S') OR $
             STRMATCH(strmid(htmlfile[i],5,1,/reverse_offset), 'N') THEN BEGIN
              event_begin=[event_begin,STRMID(htmlfile[i],37,8,/reverse_offset)]
              event_end=[event_end,STRMID(htmlfile[i],28,8,/reverse_offset)]
              event_peak=[event_peak,STRMID(htmlfile[i],19,8,/reverse_offset)]
              event_mag=[event_mag,STRMID(htmlfile[i],10,5,/reverse_offset)]
              event_loc=[event_loc,STRMID(htmlfile[i],5,6,/reverse_offset)]
              event_date=[event_date,STRMID(htmlfile[i],48,10,/reverse_offset)]

              if (STRMID(htmlfile[i],48,10,/reverse_offset) eq refdate) then begin
                  x_heliographic(j) = FIX(STRMID(htmlfile[i],4,2,/reverse_offset))
                  y_heliographic(j) = FIX(STRMID(htmlfile[i],1,2,/reverse_offset))
                  ; EAST or WEST ?
                  EW                = STRMID(htmlfile[i],2,1,/reverse_offset)
                  IF (EW eq 'E') then y_heliographic(j) = -y_heliographic(j)
                  ; NORTH or SOUTH?
                  NS                = STRMID(htmlfile[i],5,1,/reverse_offset)
                  IF (NS eq 'S') then x_heliographic(j) = -x_heliographic(j)
                  String1(j)        =     STRMID(htmlfile[i],10,5,/reverse_offset) ; Flare Type
                  String2(j)        =     STRMID(htmlfile[i],19,5,/reverse_offset) ; Peak time
                  String1(j) = String1(j) + String2(j) 
                  j=j+1
              endif
          ENDIF
      ENDFOR

      NrEntries=j 

      if (NrEntries eq 0) then goto, jump

;^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ 

; this part prevents string overwriting



; Transform Coordinates from heliographic 512x512 Pixel map

for i=0,nrEntries-1 do begin;;
    plots,y_heliographic(i),x_heliographic(i),color=200+i,Psym=3 ; read out coordinates
; get position in Window in a tricky way
    image=tvrd()
    if (i eq 0 ) then result=[WHERE(image eq 200+i,count)]
    if (i gt 0 ) then result=[result,WHERE(image eq 200+i,count)]
endfor

x=result mod 512
y=floor(result/512.)

ERASE
;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  
  flag = intarr(NrEntries)

  k=0
  
  string1=StrTrim(String1)

  for i=0,NrEntries-1 do begin
      for j=0,i-1 do begin
          
; distance between origin of Textfields d:

          dx=(x(i)-x(j))^2
          dy=(y(i)-y(j))^2
          d=sqrt(dx+dy)
 
          if (d le 15.) then begin
              
              if (flag(j) eq 0) then begin
                  k=k+1
                  flag(i) = k
                  flag(j) = flag(i)
              end
              
              
              if (flag(j) gt 0) then begin
                  flag(i) = flag(j)
              end
              
          endif
          
      endfor
  endfor

;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  
  for i=0,NrEntries-1 do begin
      if ( flag(i) eq 0 ) then flag(i) = Max(flag)+1
  endfor
  
  

  MaxFlag=max(flag)

  add  = 0
  xloc = intarr(NrEntries)
  yloc = intarr(NrEntries)
  

;  for i=1,MaxFlag do begin
     
 ; endfor


  for i=0,NrEntries-1 do begin
      
      xloc(i) = x(i)            ; round(x(i)*scalig+256.)
      yloc(i) = y(i)            ; round(y(i)*scaling+256.)

      if (flag(i) eq 0) then begin
        xyouts,xloc(i),yloc(i),color=255,charsize=1.4, String1(i),alignment=0.0,/device
      endif
      
  endfor
  
  if (MaxFlag gt 0) then begin
     FirstEntry = intarr(MaxFlag)
      x_av       = intarr(MaxFlag)
      y_av       = intarr(MaxFlag)
      FlagCount  = intarr(MaxFlag)
      FlagSymbol = STRarr(20)
      xFlag      = intarr(MaxFlag)
      yFlag      = intarr(MaxFlag)
   endif
  
   FlagSymbolColor= 255
   marker1=intarr(20)
  
   for k=1,MaxFlag do begin
       
       for i=0,NrEntries-1 do begin

                                ; define markers
           if (k eq 1) then marker1(1) = 7
           if (k eq 2) then marker1(2) = 4
           if (k eq 3) then marker1(3) = 1
           if (k eq 4) then marker1(4) = 5
           if (k eq 5) then marker1(5) = 6
           if (k eq 6) then marker1(6) = 2
           if (k eq 7) then begin
               FlagSymbolColor= 200
               marker1(k) = marker1(k-6)
           endif
  

           if (flag(i) eq k) then begin
               
               FlagCount(k-1) = FlagCount(k-1) + 1
               
               x_av(k-1) = x_av(k-1) + xloc(i)
               y_av(k-1) = y_av(k-1) + yloc(i)
  
               add  = add+1
               
               xloc(i) = 20
               yloc(i) = round(508-add*15)
               
               if (FirstEntry(k-1) gt 0 ) then begin
                   xyouts,xloc(i),yloc(i),color=255,charsize=1.,String1(i),/device ; Strings in Legend 1/2
               endif
               
               if (FirstEntry(k-1) eq 0 ) then begin
                   
                   add=add+0.5
                   yloc(i) = round(508-add*15)
                   xflag(k-1) =xloc(i)-10
                   yflag(k-1) =yloc(i)+5

                   plots,xFlag(k-1),yFlag(k-1),color=FlagSymbolColor,psym=marker1(k),/device
                   xyouts,xloc(i),yloc(i)      ,color=FlagSymbolColor,charsize=1.,String1(i)     ,/device ; Strings in Legend 2/2
                   FirstEntry(k-1) = 1
                   
               endif
               
           endif
           
       endfor
       
       x_av(k-1) = Round(x_av(k-1) / FlagCount(k-1))
       y_av(k-1) = Round(y_av(k-1) / FlagCount(k-1))
    
       plots,x_av(k-1),y_av(k-1),color=FlagSymbolColor,Psym=marker1(k) ,/device ;im Bild

   endfor
   
;^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ 


;-------- write output
   
   outfn_prefix = 'SLE'            ; Solarsoft Latest Events
   outfn   = outfn_prefix+year_4ch+moy+dom
   outdir  = !swbs.out_dir+year_4ch+'/'+moy+'/'+dom+'/'
   hdr       = 'This is the text file for the CACTus presentation in Solar Weather Browser'
   
;   cd, outdir

;----- write image:

   image=tvrd()
   data_type='O'
   data_name='SolarSoft events'
   color_table_code='000'
   date=refdate1

;----- write smffile:

   smffile =outfn+'.smf'

   openw,unit,tmpdir+smffile,/get_lun

   ;  write header
   printf,unit,'meta|svg'
   printf,unit,'# begin a page'
   printf,unit,'openpl'
   printf,unit,'# coordinate system: bottom left is 0,0, top right is 1,1'
   printf,unit,'fspace|0|0|1|1'
;   printf,unit,'bgcolorname|black'  
   printf,unit,'bgcolorname|none'  
   printf,unit,'erase'
   printf,unit,' '
   printf,unit,'# draw here'
   printf,unit,' '
           
   printf,unit,'pencolorname|white'
   printf,unit,'ffontsize|0.025'
;   printf,unit,'flinewidth|0.5'

    for i =0,NrEntries-1 do begin
        printf,unit,'fmove|',xloc(i)/512.,'|',yloc(i)/512.
        printf,unit,'pencolor|65025|65025|65025'
        printf,unit,'label|',String1(i)

    endfor

    marker = strarr(50)

    for k =1,MaxFlag do begin

        ; define markers
        if (k eq 1) then marker(1) = 5 
        if (k eq 2) then marker(2) = 4
        if (k eq 3) then marker(3) = 8
        if (k eq 4) then marker(4) = 2
        if (k eq 5) then marker(5) = 6 
        if (k eq 6) then marker(6) = 7 
        if (k eq 7) then marker(7) = 9 
        if (k eq 8) then marker(8) = 3 
        if (k eq 9) then marker(9) = 19
        if (k eq 10) then marker(10) = 20
        if (k ge 11) then marker(k) = k

        ;;;Draw Shadow
	printf,unit,'pencolor|10|10|10'
	printf,unit,'fmarker|',x_av(k-1) /512.,'|', (y_av(k-1)-1)/512.,'|',marker(k),'|0.03'
	printf,unit,'fmarker|',x_av(k-1) /512.,'|', (y_av(k-1)+1)/512.,'|',marker(k),'|0.03'
	printf,unit,'fmarker|',(x_av(k-1)-1) /512.,'|', y_av(k-1)/512.,'|',marker(k),'|0.03'
	printf,unit,'fmarker|',(x_av(k-1)+1) /512.,'|', y_av(k-1)/512.,'|',marker(k),'|0.03'
	;printf,unit,'fmarker|',xFlag(k-1)/512.,'|',yFlag(k-1)/512.,'|',marker(k),'|0.03'
	;;;;
        printf,unit,'pencolor|65025|65025|65025'
	printf,unit,'fmarker|',x_av(k-1) /512.,'|', y_av(k-1)/512.,'|',marker(k),'|0.03'
        printf,unit,'fmarker|',xFlag(k-1)/512.,'|',yFlag(k-1)/512.,'|',marker(k),'|0.03'

   endfor


          ;  write end
   printf,unit,'# end page'
   printf,unit,'closepl'
           
   close, unit
   free_Lun,unit
;----convert file
smf2svg,tmpdir+outfn+'.smf',tmpdir+outfn+'.svg'
;----------------
    write2zip, outfn, outdir, image, hdr, data_type,data_name, color_table_code, date,$
      /logfile,/svg


;@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@


jump:

ENDFOR

;----------------------------------------------------------- 

 ; say bye-bye
   spawn, 'date', fulldate     
   printf, log, ' <<<< '+myname+' finished at '+fulldate 
 ; close log file if needed   
   IF keyword_set(logfile) THEN BEGIN
      close, log
      free_lun, log
   ENDIF
END


